/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Dict
 *
 * The Initial Developer of the Original Code is David Costanzo
 * Portions created by the Initial Developer are Copyright (C) 2003-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/////////////////////////////////////////////////////////////////////////////
// HACK: this block of code was pref.js. 
// For some reason we can't include it from context.xul
// 
const PREFNAME_DEFAULTSERVERNAME = "extensions.dict.defaultservername";
const PREFNAME_DEFAULTSERVERPORT = "extensions.dict.defaultserverport";

// Get the default server name, or use a default value
// Does not throw an exception.
function dictGetDefaultServerName() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return 'dict.org';
}

// Get the default server port, or use a default value
// Does not throw an exception.
function dictGetDefaultServerPort() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    try {
        // maybe it's stored as an int
        return pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return '2628';
}
// END HACK
/////////////////////////////////////////////////////////////////////////////


// DictListener class, which is passed in as the "listener" parameter
// to DictClient::defineWord().
function DictListener () {
}

// wraps some HTML inside HTML/BODY tag and adds a title
DictListener.prototype = 
{

    createHtmlDocument : function (title, body)
    {
        var htmlDocument =
            '<HTML>\n'    +
            '  <HEAD>\n'  + 
            '      <TITLE>' + dictUtf8ToCharacterEntities(title) + '</TITLE>\n' +
            '      <SCRIPT type="text/javascript">\n' + 

            // This HTML must be self-resizing.  Doing it from the code that
            // creates the window doesn't work with data: URIs.
            'function load() {\n' +
            '    window.sizeToContent();\n' +
            '    if (window.outerWidth < window.innerWidth) {\n' +

                     // The window became narrower than the content
                     // This should never happen, but it may be happening
                     // because of bug #7097--a bug in DownloadStatusBar
                     // when it has over 40 items.
                     // I cannot fix DownloadStatusBar, but I can be defensive
                     // and resize the window back to its original width.
            '        window.outerWidth = window.innerWidth;\n' +
            '    }\n' +

                 // If the page is too long to display without scrollbars, then shorten it
            '    var proposedHeight = window.innerHeight + 70;\n' + 
            '    if (600 < proposedHeight) {\n' +
            '        window.innerHeight = 600;\n' + 

                     // Make the window wider to account for the width of the
                     // vertical scrollbar.  If we don't do this, a 
                     // horizontal scrollbar will appear.
            '        window.innerWidth += 25;\n' +
            '    }\n' +

            '}\n' +
            '      </SCRIPT>\n' +
            '  </HEAD>\n' +
            '  <BODY class="dict-page" onload="load();">\n' +
            dictUtf8ToCharacterEntities(body) +
            '  </BODY>\n' +
            '</HTML>\n';
        
        return htmlDocument;
    },


    //
    // The next three functions implement the DictListener interface
    //

    // Called when the DictClient has definitions
    onHaveDefinitions : function (word, definitions)
    {
        // Format the definitions into HTML
        var htmlBody = '';

        // count the number of sources -- used for formatting
        var totalSources = 0;
        for (source in definitions) {
            totalSources++;
        }

        var sourceIndex = 0;
        for (source in definitions) {

            htmlBody += '<B><DIV class="dict-database">';
            htmlBody += source;
            htmlBody += '</DIV></B>\n';

            htmlBody += '<CODE><SPAN class="dict-definition">\n';

            var i;
            for (i = 0; i < definitions[source].length; i++) {

                var deftext = definitions[source][i].text;

                // normalize all definitions so that they end in a single newline
                deftext = deftext.replace(/\s*$/, '\n');

                // Escape HTML entities
                deftext = deftext.replace(/&/g, '&amp;');
                deftext = deftext.replace(/</g, '&lt;');

                // replace newlines with <BR> and whitespace with
                // non-breaking spaces to preserve formatting.
                // This is better than using <PRE> because it
                // allows us to use other mark-up tags.
                deftext = deftext.replace(/\n/g, '<BR>');
                deftext = deftext.replace(/ /g, '&nbsp;');

                // Words that are cross-referenced by WordNet show up 
                // in braces. e.g., "{word}" and "{word(P)}"
                deftext = deftext.replace(/{(.*?)(\(.*\))?}/g, '$1');

                // add a newline after each <BR> to make the 
                // HTML source more readable.
                deftext = deftext.replace(/<BR>/g, '<BR>\n');

                // append an extra newline after everything but the
                // the final definition from the final source.
                // (Otherwise there'd be a strange-looking blank space
                // at the bottom of the page.)
                if (sourceIndex + 1 != totalSources ||
                    i + 1           != definitions[source].length) {
                    deftext += '<BR>\n';
                }

                htmlBody += deftext;
            }

            htmlBody += '</SPAN></CODE>\n';
            sourceIndex++;
        }

        // create an HTML document
        var htmlDocument = this.createHtmlDocument(word, htmlBody);

        // create a window for the HTML document
        const features = [
            'status=no', 
            'toolbar=no',
            'menubar=no',
            'location=no',
            'scrollbars=yes',
            'resizable=yes',
            'width=475',
            'height=100'
        ];

        // open the popup window
        var popupWindow;

        try {
            // Use XPCNativeWrapper to fix a potential security vulnerability
            // of calling window.open() that overridden by a malicious entity.
            //
            // This also works around an incompatability with 
            // Tabbrowser Extension's "single window mode" where the
            // window.open() that it provides does not have a document
            // property.
            var winWrapper = new XPCNativeWrapper(window, 'open()');
            popupWindow = winWrapper.open(
                'data:text/html,' + dictUtf8ToCharacterEntities(htmlDocument),
                '_blank',
                features.join(','));
        }
        catch (e) {

            try {
                popupWindow = window.open(
                    'data:text/html,' + dictUtf8ToCharacterEntities(htmlDocument),
                    '',
                    features.join(','));
            }
            catch (e) {
                // For some reason, window.open() throws
                // an exception on Thunderbird on Windows 95 if
                // the "chrome" flag is missing.
                // Because the chrome flag makes the window non-selectable
                // we only want to specify it if absolutely necessary.
                popupWindow = window.open(
                    'data:text/html,' + dictUtf8ToCharacterEntities(htmlDocument),
                    '',
                    'chrome,' + features.join(','));
            }
        }
    },

    // Called when the DictClient has closest matches
    onHaveClosestMatches : function (word, matches)
    {
        // There is more than one closest match
        // Display them all and let the user decide which to define
        var matchesArray = [];
        for (match in matches) {
            matchesArray.push(match);
        }

        var query = word + '|' + matchesArray.join('|');

        // open the chrome window
        window.open(
            'chrome://dict/content/close-matches.xul?' + escape(query),
            '',
            'chrome,scrollbars,resizable');
    },

    onNoDefinitionsFound : function (word) 
    {
        alert('No definitions were found for "' + dictUtf8ToUcs4(word) + '".\n');
    },
    
    onError : function (error)
    {
        alert(dictUtf8ToUcs4(error));
    }
}


function dictDefineWord(word)
{
    if (word.indexOf('"') != -1) {
        alert("Sorry, selections that contain quotation marks cannot be defined.");
        return;
    }

    if (word.length == 0) {
        alert("Please select a word to define.");
        return;
    }

    try {
        // read the host and port from the preferences
        var host = dictGetDefaultServerName();
        var port = dictGetDefaultServerPort();

        var listener = new DictListener();
        var client   = new DictClient();
        client.defineWord(host, port, word, listener);
    }
    catch (ex) {
        alert(ex);
    }
}
